@@ -240,8 +240,18 @@ public class Version {
     public static final Version V_1_6_1 = new Version(V_1_6_1_ID, false, org.apache.lucene.util.Version.LUCENE_4_10_4);
     public static final int V_1_6_2_ID = 1060299;
     public static final Version V_1_6_2 = new Version(V_1_6_2_ID, false, org.apache.lucene.util.Version.LUCENE_4_10_4);
+    public static final int V_1_6_3_ID = 1060399;
+    public static final Version V_1_6_3 = new Version(V_1_6_3_ID, true, org.apache.lucene.util.Version.LUCENE_4_10_4);
+    public static final int V_1_7_0_ID = 1070099;
+    public static final Version V_1_7_0 = new Version(V_1_7_0_ID, false, org.apache.lucene.util.Version.LUCENE_4_10_4);
+    public static final int V_1_7_1_ID = 1070199;
+    public static final Version V_1_7_1 = new Version(V_1_7_1_ID, false, org.apache.lucene.util.Version.LUCENE_4_10_4);
+    public static final int V_1_7_2_ID = 1070299;
+    public static final Version V_1_7_2 = new Version(V_1_7_2_ID, false, org.apache.lucene.util.Version.LUCENE_4_10_4);
+    public static final int V_1_7_3_ID = 1070399;
+    public static final Version V_1_7_3 = new Version(V_1_7_3_ID, false, org.apache.lucene.util.Version.LUCENE_4_10_4);
 
-    public static final Version CURRENT = V_1_6_2;
+    public static final Version CURRENT = V_1_7_3;
 
     static {
         assert CURRENT.luceneVersion.equals(Lucene.VERSION) : "Version must be upgraded to [" + Lucene.VERSION + "] is still set to [" + CURRENT.luceneVersion + "]";
@@ -253,6 +263,16 @@ public class Version {
 
     public static Version fromId(int id) {
         switch (id) {
+            case V_1_7_3_ID:
+                return V_1_7_3;
+            case V_1_7_2_ID:
+                return V_1_7_2;
+            case V_1_7_1_ID:
+                return V_1_7_1;
+            case V_1_7_0_ID:
+                return V_1_7_0;
+            case V_1_6_3_ID:
+                return V_1_6_3;
             case V_1_6_2_ID:
                 return V_1_6_2;
             case V_1_6_1_ID:
